/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.dom;

/**
 * A element tree which can be transformed into a display string.
 *
 * @author Torindo Nesci
 */
public interface ElementTreeDisplayable {

  /**
   * Sets display mode.
   *
   * @param displayMode  The display mode.
   */
  void setDisplayMode(HtmlNodeDisplayMode displayMode);
  
  /**
   * Returns the display mode.
   *
   * @return The display mode.
   */
  HtmlNodeDisplayMode getDisplayMode();
  
  /**
   * Clears all the element tree displayable attributes.
   */
  void removeAllDisplayAttributes();
  
  /**
   * Adds an attribute to the element tree displayable attributes.
   *
   * @param attrName The attribute name.
   * @param attrValue The attribute value.
   */
  void setDisplayAttribute(String attrName, String attrValue);
  
  /**
   * Removed the given attribute from element tree displayable attributes.
   * If the given attribute does not exist no error is thrown.
   *
   * @param attrName The attribute name.
   */
  void removeDisplayAttribute(String attrName);
  
  /**
   * Transforms the node into a string representing the node and it's children.
   *
   * @return The display string.
   */
  String toDisplayString();
  
}
