/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.histogram;

import com.flamenk.dom.HtmlNode;
import com.google.common.base.Preconditions;

/**
 * A html token.
 *
 * <br/><br/> <b>This implementation is Thread Safe if the {@link Token#getToken()}
 * subclass implementation is Thread Safe.</b>
 *
 * @author Torind Nesci
 */
public abstract class Token {
  
  private final String mTokenName;
  private final HtmlNode mHtmlNode;
  
  Token(String tokenName, HtmlNode htmlNode) {
    Preconditions.checkNotNull(tokenName);
    Preconditions.checkNotNull(htmlNode);
    
    this.mTokenName = tokenName;
    this.mHtmlNode = htmlNode;
  }
 
  /**
   * Returns the name of this token.
   *
   * @return The token's name.
   * @see com.flamenk.TagConstants
   */
  public String getName() {
    return this.mTokenName;
  }
  
  /**
   * Returns the {@link HtmlNode} this token belongs to.
   *
   * @return The {@link HtmlNode}.
   */
  public HtmlNode getHtmlNode() {
    return this.mHtmlNode;
  }
  
  /**
   * Returns a string representation of this token.
   *
   * @return The string representation of this token.
   */
  protected abstract String getToken();
  
  /**
   * Returns a string representation of this token.
   * It uses {@link Token#getToken()}.
   *
   * @return The string representation of this token.
   */
  @Override
  public String toString() {
    return this.getToken();
  }
}
